import { defineComponent, reactive, ref, toRefs } from 'vue'
import { Button, Slider } from 'vant'
import classnames from 'classnames'
import detailState from '/src/pages/detail/state'
import styles from './speed.module.less'

import iconMinus from './icons/icon-minus.svg'
import iconPlus from './icons/icon-plus.svg'
import iconresetMinus from './icons/icon-resetMinus.svg'

const state = reactive({
  speed: 90,
  showHint: false,
})

export default defineComponent({
  name: 'speed',
  props: {
    changed: {
      type: Function,
      default: (speed: number) => {},
    },
    updateSpeed: {
      type: Function,
      default: (speed: number) => {},
    },
    mode: {
      type: String,
    },
    changeMode: {
      type: Function,
      default: (val: string) => {},
    },
    lib: {
      type: Object,
    },
  },
  setup(props) {
    const { lib } = toRefs(props)
    state.speed = lib.value?.speed
    const onDragStart = () => {
      state.showHint = true
    }

    const onDragEnd = () => {
      state.showHint = false
    }

    const changeSpeed = (speed: number) => {
      state.speed = speed
      props.changed(state.speed)
    }

    const updateSpeed = (speed: number) => {
      state.speed = speed
      props.updateSpeed(state.speed)
    }

    const minusSpeed = () => {
      state.speed = Math.max(state.speed - 1, 45)
      props.changed(state.speed)
    }

    const plusSpeed = () => {
      state.speed = Math.min(state.speed + 1, 270)
      props.changed(state.speed)
    }

    return () => (
      <div class={styles['speed-change']}>
        <div class={styles.speed}>
          {/* <span>速度</span> */}
          <div>
            <Button
              color="#01C1B5"
              icon={iconPlus}
              type="primary"
              size="mini"
              round
              disabled={state.speed == 270}
              onClick={plusSpeed}
            />
            <Slider
              class={styles.slider}
              max={270}
              min={45}
              barHeight="3px"
              buttonSize="18px"
              modelValue={315 - state.speed}
              onUpdate:modelValue={(speed: number) => updateSpeed(315 - speed)}
              step="1"
              vertical
              activeColor="#ebedf0"
              onChange={(speed: number) => changeSpeed(315 - speed)}
              onDragStart={onDragStart}
              onDragEnd={onDragEnd}
              vSlots={{
                button: () => (
                  <div class={styles['custom-button']}>
                    <span>{state.speed}</span>
                    <div class={classnames(styles.title)}>
                      <span class={styles.strong}>{state.speed}</span>
                    </div>
                  </div>
                ),
              }}
            />
            <Button
              class={styles.min}
              color="#01C1B5"
              icon={iconMinus}
              type="primary"
              size="mini"
              round
              disabled={state.speed == 45}
              onClick={minusSpeed}
            />
            <Button
              class={styles.reset}
              color="#01C1B5"
              icon={iconresetMinus}
              type="primary"
              size="mini"
              round
              onClick={() => {
                if (detailState.activeDetail) {
                  changeSpeed(detailState.activeDetail?.originalSpeed)
                }
              }}
            />
          </div>
        </div>
      </div>
    )
  },
  methods: {
    resetSpeed() {
      state.speed = this.lib?.speed
      this.changed(this.lib?.speed)
    },
    refUpdateSpeed(speed: number) {
      state.speed = speed
      this.updateSpeed(state.speed)
    },
  },
})
